@@ -30,16 +30,14 @@ import javax.servlet.RequestDispatcher;
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletResponse;
 
-import org.apache.catalina.Globals;
-import org.apache.catalina.Manager;
 import org.apache.catalina.Realm;
 import org.apache.catalina.Session;
 import org.apache.catalina.connector.Request;
-import org.apache.catalina.connector.Response;
 import org.apache.catalina.deploy.LoginConfig;
 import org.apache.coyote.ActionCode;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.buf.CharChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
@@ -52,13 +50,13 @@ import org.apache.tomcat.util.http.MimeHeaders;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: FormAuthenticator.java 1417891 2012-12-06 13:58:59Z kkolinko $
+ * @version $Id: FormAuthenticator.java 1162933 2011-08-29 18:30:11Z markt $
  */
 
 public class FormAuthenticator
     extends AuthenticatorBase {
     
-    private static Log log = LogFactory.getLog(FormAuthenticator.class);
+    private static final Log log = LogFactory.getLog(FormAuthenticator.class);
 
     // ----------------------------------------------------- Instance Variables
 
@@ -76,6 +74,12 @@ public class FormAuthenticator
      */
     protected String characterEncoding = null;
 
+    /**
+     * Landing page to use if a user tries to access the login page directly or
+     * if the session times out during login. If not set, error responses will
+     * be sent instead.
+     */
+    protected String landingPage = null;
 
     // ------------------------------------------------------------- Properties
 
@@ -83,6 +87,7 @@ public class FormAuthenticator
     /**
      * Return descriptive information about this Valve implementation.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -106,6 +111,22 @@ public class FormAuthenticator
     }
 
 
+    /**
+     * Return the landing page to use when FORM auth is mis-used.
+     */
+    public String getLandingPage() {
+        return landingPage;
+    }
+
+
+    /**
+     * Set the landing page to use when the FORM auth is mis-used.
+     */
+    public void setLandingPage(String landingPage) {
+        this.landingPage = landingPage;
+    }
+
+
     // --------------------------------------------------------- Public Methods
 
 
@@ -122,8 +143,9 @@ public class FormAuthenticator
      *
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public boolean authenticate(Request request,
-                                Response response,
+                                HttpServletResponse response,
                                 LoginConfig config)
         throws IOException {
 
@@ -223,7 +245,6 @@ public class FormAuthenticator
         uriCC.setLimit(-1);
         String contextPath = request.getContextPath();
         String requestURI = request.getDecodedRequestURI();
-        response.setContext(request.getContext());
 
         // Is this the action request from the login page?
         boolean loginAction =
@@ -249,7 +270,7 @@ public class FormAuthenticator
 
         // Yes -- Acknowledge the request, validate the specified credentials
         // and redirect to the error page if they are not correct
-        response.sendAcknowledgement();
+        request.getResponse().sendAcknowledgement();
         Realm realm = context.getRealm();
         if (characterEncoding != null) {
             request.setCharacterEncoding(characterEncoding);
@@ -273,8 +294,20 @@ public class FormAuthenticator
             if (containerLog.isDebugEnabled())
                 containerLog.debug
                     ("User took so long to log on the session expired");
-            response.sendError(HttpServletResponse.SC_REQUEST_TIMEOUT,
-                               sm.getString("authenticator.sessionExpired"));
+            if (landingPage == null) {
+                response.sendError(HttpServletResponse.SC_REQUEST_TIMEOUT,
+                        sm.getString("authenticator.sessionExpired"));
+            } else {
+                // Make the authenticator think the user originally requested
+                // the landing page
+                String uri = request.getContextPath() + landingPage;
+                SavedRequest saved = new SavedRequest();
+                saved.setMethod("GET");
+                saved.setRequestURI(uri);
+                request.getSessionInternal(true).setNote(
+                        Constants.FORM_REQUEST_NOTE, saved);
+                response.sendRedirect(response.encodeRedirectURL(uri));
+            }
             return (false);
         }
 
@@ -291,8 +324,19 @@ public class FormAuthenticator
         if (log.isDebugEnabled())
             log.debug("Redirecting to original '" + requestURI + "'");
         if (requestURI == null)
-            response.sendError(HttpServletResponse.SC_BAD_REQUEST,
-                               sm.getString("authenticator.formlogin"));
+            if (landingPage == null) {
+                response.sendError(HttpServletResponse.SC_BAD_REQUEST,
+                        sm.getString("authenticator.formlogin"));
+            } else {
+                // Make the authenticator think the user originally requested
+                // the landing page
+                String uri = request.getContextPath() + landingPage;
+                SavedRequest saved = new SavedRequest();
+                saved.setMethod("GET");
+                saved.setRequestURI(uri);
+                session.setNote(Constants.FORM_REQUEST_NOTE, saved);
+                response.sendRedirect(response.encodeRedirectURL(uri));
+            }
         else
             response.sendRedirect(response.encodeRedirectURL(requestURI));
         return (false);
@@ -300,6 +344,12 @@ public class FormAuthenticator
     }
 
 
+    @Override
+    protected String getAuthMethod() {
+        return Constants.FORM_METHOD;
+    }
+
+
     // ------------------------------------------------------ Protected Methods
 
 
@@ -307,50 +357,41 @@ public class FormAuthenticator
      * Called to forward to the login page
      * 
      * @param request Request we are processing
-     * @param response Response we are creating
+     * @param response Response we are populating
      * @param config    Login configuration describing how authentication
      *              should be performed
      * @throws IOException  If the forward to the login page fails and the call
-     *                      to {@link HttpServletResponse#sendError(int, String)
+     *                      to {@link HttpServletResponse#sendError(int, String)}
      *                      throws an {@link IOException}
      */
-    protected void forwardToLoginPage(Request request, Response response,
-            LoginConfig config) throws IOException {
+    protected void forwardToLoginPage(Request request,
+            HttpServletResponse response, LoginConfig config)
+            throws IOException {
         
-        if (log.isDebugEnabled()) {
-            log.debug(sm.getString("formAuthenticator.forwardLogin",
-                    request.getRequestURI(), request.getMethod(),
-                    config.getLoginPage(), context.getName()));
+        String loginPage = config.getLoginPage();
+        if (loginPage == null || loginPage.length() == 0) {
+            String msg = sm.getString("formAuthenticator.noLoginPage",
+                    context.getName());
+            log.warn(msg);
+            response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
+                    msg);
+            return;
         }
-
-        if (getChangeSessionIdOnAuthentication()) {
-            Session session = request.getSessionInternal(false);
-            if (session != null) {
-                Manager manager = request.getContext().getManager();
-                manager.changeSessionId(session);
-                request.changeSessionId(session.getId());
-            }
-        }
-
-        // Always use GET for the login page, regardless of the method used
-        String oldMethod = request.getCoyoteRequest().method().toString();
-        request.getCoyoteRequest().method().setString("GET");
-
+        
         RequestDispatcher disp =
-            context.getServletContext().getRequestDispatcher
-            (config.getLoginPage());
+            context.getServletContext().getRequestDispatcher(loginPage);
         try {
-            disp.forward(request.getRequest(), response.getResponse());
-            response.finishResponse();
+            if (context.fireRequestInitEvent(request)) {
+                disp.forward(request.getRequest(), response);
+                context.fireRequestDestroyEvent(request);
+            }
         } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
             String msg = sm.getString("formAuthenticator.forwardLoginFail");
             log.warn(msg, t);
-            request.setAttribute(Globals.EXCEPTION_ATTR, t);
+            request.setAttribute(RequestDispatcher.ERROR_EXCEPTION, t);
             response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
                     msg);
-        } finally {
-            // Restore original method so that it is written into access log
-            request.getCoyoteRequest().method().setString(oldMethod);
         }
     }
 
@@ -359,24 +400,40 @@ public class FormAuthenticator
      * Called to forward to the error page
      * 
      * @param request Request we are processing
-     * @param response Response we are creating
+     * @param response Response we are populating
      * @param config    Login configuration describing how authentication
      *              should be performed
      * @throws IOException  If the forward to the error page fails and the call
-     *                      to {@link HttpServletResponse#sendError(int, String)
+     *                      to {@link HttpServletResponse#sendError(int, String)}
      *                      throws an {@link IOException}
      */
-    protected void forwardToErrorPage(Request request, Response response,
-            LoginConfig config) throws IOException {
+    protected void forwardToErrorPage(Request request,
+            HttpServletResponse response, LoginConfig config)
+            throws IOException {
+        
+        String errorPage = config.getErrorPage();
+        if (errorPage == null || errorPage.length() == 0) {
+            String msg = sm.getString("formAuthenticator.noErrorPage",
+                    context.getName());
+            log.warn(msg);
+            response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
+                    msg);
+            return;
+        }
+
         RequestDispatcher disp =
             context.getServletContext().getRequestDispatcher
             (config.getErrorPage());
         try {
-            disp.forward(request.getRequest(), response.getResponse());
+            if (context.fireRequestInitEvent(request)) {
+                disp.forward(request.getRequest(), response);
+                context.fireRequestDestroyEvent(request);
+            }
         } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
             String msg = sm.getString("formAuthenticator.forwardErrorFail");
             log.warn(msg, t);
-            request.setAttribute(Globals.EXCEPTION_ATTR, t);
+            request.setAttribute(RequestDispatcher.ERROR_EXCEPTION, t);
             response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
                     msg);
         }
@@ -385,7 +442,7 @@ public class FormAuthenticator
 
     /**
      * Does this request match the saved one (so that it must be the redirect
-     * we signalled after successful authentication?
+     * we signaled after successful authentication?
      *
      * @param request The request to be verified
      */
@@ -407,11 +464,11 @@ public class FormAuthenticator
           return (false);
 
       // Does the request URI match?
-      String decodedRequestURI = request.getDecodedRequestURI();
-      if (decodedRequestURI == null) {
+      String requestURI = request.getRequestURI();
+      if (requestURI == null)
           return (false);
-      }
-      return (decodedRequestURI.equals(sreq.getDecodedRequestURI()));
+      return (requestURI.equals(sreq.getRequestURI()));
+
     }
 
 
@@ -425,7 +482,7 @@ public class FormAuthenticator
      * @param session The session containing the saved information
      */
     protected boolean restoreRequest(Request request, Session session)
-        throws IOException {
+            throws IOException {
 
         // Retrieve and remove the SavedRequest object from our session
         SavedRequest saved = (SavedRequest)
@@ -437,60 +494,68 @@ public class FormAuthenticator
 
         // Modify our current request to reflect the original one
         request.clearCookies();
-        Iterator cookies = saved.getCookies();
+        Iterator<Cookie> cookies = saved.getCookies();
         while (cookies.hasNext()) {
-            request.addCookie((Cookie) cookies.next());
+            request.addCookie(cookies.next());
         }
 
-        String method = saved.getMethod();
         MimeHeaders rmh = request.getCoyoteRequest().getMimeHeaders();
         rmh.recycle();
-        boolean cachable = "GET".equalsIgnoreCase(method) ||
-                           "HEAD".equalsIgnoreCase(method);
-        Iterator names = saved.getHeaderNames();
+        boolean cachable = "GET".equalsIgnoreCase(saved.getMethod()) ||
+                           "HEAD".equalsIgnoreCase(saved.getMethod());
+        Iterator<String> names = saved.getHeaderNames();
         while (names.hasNext()) {
-            String name = (String) names.next();
+            String name = names.next();
             // The browser isn't expecting this conditional response now.
             // Assuming that it can quietly recover from an unexpected 412.
             // BZ 43687
             if(!("If-Modified-Since".equalsIgnoreCase(name) ||
                  (cachable && "If-None-Match".equalsIgnoreCase(name)))) {
-                Iterator values = saved.getHeaderValues(name);
+                Iterator<String> values = saved.getHeaderValues(name);
                 while (values.hasNext()) {
-                    rmh.addValue(name).setString( (String)values.next() );
+                    rmh.addValue(name).setString(values.next());
                 }
             }
         }
         
         request.clearLocales();
-        Iterator locales = saved.getLocales();
+        Iterator<Locale> locales = saved.getLocales();
         while (locales.hasNext()) {
-            request.addLocale((Locale) locales.next());
+            request.addLocale(locales.next());
         }
         
         request.getCoyoteRequest().getParameters().recycle();
         request.getCoyoteRequest().getParameters().setQueryStringEncoding(
                 request.getConnector().getURIEncoding());
 
-        ByteChunk body = saved.getBody();
-
-        if (body != null) {
-            request.getCoyoteRequest().action
-                (ActionCode.ACTION_REQ_SET_BODY_REPLAY, body);
-
-            // Set content type
-            MessageBytes contentType = MessageBytes.newInstance();
-            
-            //If no content type specified, use default for POST
-            String savedContentType = saved.getContentType();
-            if (savedContentType == null && "POST".equalsIgnoreCase(method)) {
-                savedContentType = "application/x-www-form-urlencoded";
-            }
-
-            contentType.setString(savedContentType);
-            request.getCoyoteRequest().setContentType(contentType);
+        // Swallow any request body since we will be replacing it
+        byte[] buffer = new byte[4096];
+        InputStream is = request.getInputStream();
+        while (is.read(buffer) >= 0) {
+            // Ignore request body
         }
-        request.getCoyoteRequest().method().setString(method);
+        
+        if ("POST".equalsIgnoreCase(saved.getMethod())) {
+            ByteChunk body = saved.getBody();
+            
+            if (body != null) {
+                request.getCoyoteRequest().action
+                    (ActionCode.REQ_SET_BODY_REPLAY, body);
+    
+                // Set content type
+                MessageBytes contentType = MessageBytes.newInstance();
+                
+                //If no content type specified, use default for POST
+                String savedContentType = saved.getContentType();
+                if (savedContentType == null) {
+                    savedContentType = "application/x-www-form-urlencoded";
+                }
+
+                contentType.setString(savedContentType);
+                request.getCoyoteRequest().setContentType(contentType);
+            }
+        }
+        request.getCoyoteRequest().method().setString(saved.getMethod());
 
         request.getCoyoteRequest().queryString().setString
             (saved.getQueryString());
@@ -519,37 +584,35 @@ public class FormAuthenticator
             for (int i = 0; i < cookies.length; i++)
                 saved.addCookie(cookies[i]);
         }
-        Enumeration names = request.getHeaderNames();
+        Enumeration<String> names = request.getHeaderNames();
         while (names.hasMoreElements()) {
-            String name = (String) names.nextElement();
-            Enumeration values = request.getHeaders(name);
+            String name = names.nextElement();
+            Enumeration<String> values = request.getHeaders(name);
             while (values.hasMoreElements()) {
-                String value = (String) values.nextElement();
+                String value = values.nextElement();
                 saved.addHeader(name, value);
             }
         }
-        Enumeration locales = request.getLocales();
+        Enumeration<Locale> locales = request.getLocales();
         while (locales.hasMoreElements()) {
-            Locale locale = (Locale) locales.nextElement();
+            Locale locale = locales.nextElement();
             saved.addLocale(locale);
         }
 
-        // May need to acknowledge a 100-continue expectation
-        request.getResponse().sendAcknowledgement();
+        if ("POST".equalsIgnoreCase(request.getMethod())) {
+            // May need to acknowledge a 100-continue expectation
+            request.getResponse().sendAcknowledgement();
 
-        ByteChunk body = new ByteChunk();
-        body.setLimit(request.getConnector().getMaxSavePostSize());
+            ByteChunk body = new ByteChunk();
+            body.setLimit(request.getConnector().getMaxSavePostSize());
 
-        byte[] buffer = new byte[4096];
-        int bytesRead;
-        InputStream is = request.getInputStream();
-    
-        while ( (bytesRead = is.read(buffer) ) >= 0) {
-            body.append(buffer, 0, bytesRead);
-        }
-
-        // Only save the request body if there is something to save
-        if (body.getLength() > 0) {
+            byte[] buffer = new byte[4096];
+            int bytesRead;
+            InputStream is = request.getInputStream();
+        
+            while ( (bytesRead = is.read(buffer) ) >= 0) {
+                body.append(buffer, 0, bytesRead);
+            }
             saved.setContentType(request.getContentType());
             saved.setBody(body);
         }
@@ -557,7 +620,6 @@ public class FormAuthenticator
         saved.setMethod(request.getMethod());
         saved.setQueryString(request.getQueryString());
         saved.setRequestURI(request.getRequestURI());
-        saved.setDecodedRequestURI(request.getDecodedRequestURI());
 
         // Stash the SavedRequest in our session for later use
         session.setNote(Constants.FORM_REQUEST_NOTE, saved);
@@ -577,7 +639,7 @@ public class FormAuthenticator
             (SavedRequest) session.getNote(Constants.FORM_REQUEST_NOTE);
         if (saved == null)
             return (null);
-        StringBuffer sb = new StringBuffer(saved.getRequestURI());
+        StringBuilder sb = new StringBuilder(saved.getRequestURI());
         if (saved.getQueryString() != null) {
             sb.append('?');
             sb.append(saved.getQueryString());
