@@ -18,7 +18,6 @@
 
 package org.apache.catalina.manager;
 
-import java.io.IOException;
 import java.io.PrintWriter;
 import java.lang.reflect.Method;
 import java.text.MessageFormat;
@@ -31,10 +30,10 @@ import java.util.Vector;
 import javax.management.MBeanServer;
 import javax.management.ObjectInstance;
 import javax.management.ObjectName;
-import javax.servlet.ServletException;
 import javax.servlet.http.HttpServletResponse;
 
 import org.apache.catalina.util.RequestUtil;
+import org.apache.tomcat.util.ExceptionUtils;
 
 /**
  * This is a refactoring of the servlet to externalize
@@ -42,7 +41,7 @@ import org.apache.catalina.util.RequestUtil;
  * use XSLT, that is unnecessarily complex.
  *
  * @author Peter Lin
- * @version $Id: StatusTransformer.java 1057270 2011-01-10 16:43:40Z jfclere $
+ * @version $Id: StatusTransformer.java 1089638 2011-04-06 21:31:25Z rjung $
  */
 
 public class StatusTransformer {
@@ -62,22 +61,21 @@ public class StatusTransformer {
 
 
     /**
-     * Process a GET request for the specified resource.
+     * Write an HTML or XML header.
      *
-     * @param request The servlet request we are processing
-     * @param response The servlet response we are creating
+     * @param writer the PrintWriter to use
+     * @param args Path prefix for URLs
+     * @param mode - 0 = HTML header, 1 = XML declaration
      *
-     * @exception IOException if an input/output error occurs
-     * @exception ServletException if a servlet-specified error occurs
      */
-    public static void writeHeader(PrintWriter writer, int mode) {
+    public static void writeHeader(PrintWriter writer, Object[] args, int mode) {
         if (mode == 0){
             // HTML Header Section
             writer.print(Constants.HTML_HEADER_SECTION);
         } else if (mode == 1){
             writer.write(Constants.XML_DECLARATION);
-            writer.write
-                (Constants.XML_STYLE);
+            writer.print(MessageFormat.format
+                     (Constants.XML_STYLE, args));
             writer.write("<status>");
         }
     }
@@ -153,7 +151,7 @@ public class StatusTransformer {
         boolean ok = false;
         try {
             String methodName = "info";
-            Class paramTypes[] = new Class[1];
+            Class<?> paramTypes[] = new Class[1];
             paramTypes[0] = result.getClass();
             Object paramValues[] = new Object[1];
             paramValues[0] = result;
@@ -162,7 +160,7 @@ public class StatusTransformer {
             method.invoke(null, paramValues);
             ok = true;
         } catch (Throwable t) {
-            // Ignore
+            ExceptionUtils.handleThrowable(t);
         }
         
         if (ok) {
@@ -171,22 +169,23 @@ public class StatusTransformer {
 
                 writer.print("<p>");
                 writer.print(" Physical memory: ");
-                writer.print(formatSize(new Long(result[0]), true));
+                writer.print(formatSize(Long.valueOf(result[0]), true));
                 writer.print(" Available memory: ");
-                writer.print(formatSize(new Long(result[1]), true));
+                writer.print(formatSize(Long.valueOf(result[1]), true));
                 writer.print(" Total page file: ");
-                writer.print(formatSize(new Long(result[2]), true));
+                writer.print(formatSize(Long.valueOf(result[2]), true));
                 writer.print(" Free page file: ");
-                writer.print(formatSize(new Long(result[3]), true));
+                writer.print(formatSize(Long.valueOf(result[3]), true));
                 writer.print(" Memory load: ");
-                writer.print(new Long(result[6]));
+                writer.print(Long.valueOf(result[6]));
                 writer.print("<br>");
                 writer.print(" Process kernel time: ");
-                writer.print(formatTime(new Long(result[11] / 1000), true));
+                writer.print(formatTime(Long.valueOf(result[11] / 1000), true));
                 writer.print(" Process user time: ");
-                writer.print(formatTime(new Long(result[12] / 1000), true));
+                writer.print(formatTime(Long.valueOf(result[12] / 1000), true));
                 writer.print("</p>");
             } else if (mode == 1){
+                // NO-OP
             }
         }
         
@@ -205,14 +204,14 @@ public class StatusTransformer {
 
             writer.print("<p>");
             writer.print(" Free memory: ");
-            writer.print(formatSize
-                         (new Long(Runtime.getRuntime().freeMemory()), true));
+            writer.print(formatSize(
+                    Long.valueOf(Runtime.getRuntime().freeMemory()), true));
             writer.print(" Total memory: ");
-            writer.print(formatSize
-                         (new Long(Runtime.getRuntime().totalMemory()), true));
+            writer.print(formatSize(
+                    Long.valueOf(Runtime.getRuntime().totalMemory()), true));
             writer.print(" Max memory: ");
-            writer.print(formatSize
-                         (new Long(Runtime.getRuntime().maxMemory()), true));
+            writer.print(formatSize(
+                    Long.valueOf(Runtime.getRuntime().maxMemory()), true));
             writer.print("</p>");
         } else if (mode == 1){
             writer.write("<jvm>");
@@ -231,13 +230,10 @@ public class StatusTransformer {
     /**
      * Write connector state.
      */
-    public static void writeConnectorState(PrintWriter writer, 
-                                           ObjectName tpName, String name,
-                                           MBeanServer mBeanServer,
-                                           Vector globalRequestProcessors,
-                                           Vector requestProcessors,
-                                           int mode)
-        throws Exception {
+    public static void writeConnectorState(PrintWriter writer,
+            ObjectName tpName, String name, MBeanServer mBeanServer,
+            Vector<ObjectName> globalRequestProcessors,
+            Vector<ObjectName> requestProcessors, int mode) throws Exception {
 
         if (mode == 0) {
             writer.print("<h1>");
@@ -263,9 +259,10 @@ public class StatusTransformer {
 
             ObjectName grpName = null;
 
-            Enumeration enumeration = globalRequestProcessors.elements();
+            Enumeration<ObjectName> enumeration =
+                globalRequestProcessors.elements();
             while (enumeration.hasMoreElements()) {
-                ObjectName objectName = (ObjectName) enumeration.nextElement();
+                ObjectName objectName = enumeration.nextElement();
                 if (name.equals(objectName.getKeyProperty("name"))) {
                     grpName = objectName;
                 }
@@ -297,7 +294,7 @@ public class StatusTransformer {
 
             enumeration = requestProcessors.elements();
             while (enumeration.hasMoreElements()) {
-                ObjectName objectName = (ObjectName) enumeration.nextElement();
+                ObjectName objectName = enumeration.nextElement();
                 if (name.equals(objectName.getKeyProperty("worker"))) {
                     writer.print("<tr>");
                     writeProcessorState(writer, objectName, mBeanServer, mode);
@@ -321,9 +318,10 @@ public class StatusTransformer {
 
             ObjectName grpName = null;
 
-            Enumeration enumeration = globalRequestProcessors.elements();
+            Enumeration<ObjectName> enumeration =
+                globalRequestProcessors.elements();
             while (enumeration.hasMoreElements()) {
-                ObjectName objectName = (ObjectName) enumeration.nextElement();
+                ObjectName objectName = enumeration.nextElement();
                 if (name.equals(objectName.getKeyProperty("name"))) {
                     grpName = objectName;
                 }
@@ -343,7 +341,7 @@ public class StatusTransformer {
                 writer.write("<workers>");
                 enumeration = requestProcessors.elements();
                 while (enumeration.hasMoreElements()) {
-                    ObjectName objectName = (ObjectName) enumeration.nextElement();
+                    ObjectName objectName = enumeration.nextElement();
                     if (name.equals(objectName.getKeyProperty("worker"))) {
                         writeProcessorState(writer, objectName, mBeanServer, mode);
                     }
@@ -552,7 +550,7 @@ public class StatusTransformer {
 
         if (mode == 0){
             ObjectName queryHosts = new ObjectName("*:j2eeType=WebModule,*");
-            Set hostsON = mBeanServer.queryNames(queryHosts, null);
+            Set<ObjectName> hostsON = mBeanServer.queryNames(queryHosts, null);
 
             // Navigation menu
             writer.print("<h1>");
@@ -561,9 +559,9 @@ public class StatusTransformer {
 
             writer.print("<p>");
             int count = 0;
-            Iterator iterator = hostsON.iterator();
+            Iterator<ObjectName> iterator = hostsON.iterator();
             while (iterator.hasNext()) {
-                ObjectName contextON = (ObjectName) iterator.next();
+                ObjectName contextON = iterator.next();
                 String webModuleName = contextON.getKeyProperty("name");
                 if (webModuleName.startsWith("//")) {
                     webModuleName = webModuleName.substring(2);
@@ -588,7 +586,7 @@ public class StatusTransformer {
             count = 0;
             iterator = hostsON.iterator();
             while (iterator.hasNext()) {
-                ObjectName contextON = (ObjectName) iterator.next();
+                ObjectName contextON = iterator.next();
                 writer.print("<a class=\"A.name\" name=\"" 
                              + (count++) + ".0\">");
                 writeContext(writer, contextON, mBeanServer, mode);
@@ -630,19 +628,21 @@ public class StatusTransformer {
             }
 
             ObjectName queryManager = new ObjectName
-                (objectName.getDomain() + ":type=Manager,path=" + contextName 
+                (objectName.getDomain() + ":type=Manager,context=" + contextName 
                  + ",host=" + hostName + ",*");
-            Set managersON = mBeanServer.queryNames(queryManager, null);
+            Set<ObjectName> managersON =
+                mBeanServer.queryNames(queryManager, null);
             ObjectName managerON = null;
-            Iterator iterator2 = managersON.iterator();
+            Iterator<ObjectName> iterator2 = managersON.iterator();
             while (iterator2.hasNext()) {
-                managerON = (ObjectName) iterator2.next();
+                managerON = iterator2.next();
             }
 
             ObjectName queryJspMonitor = new ObjectName
                 (objectName.getDomain() + ":type=JspMonitor,WebModule=" +
                  webModuleName + ",*");
-            Set jspMonitorONs = mBeanServer.queryNames(queryJspMonitor, null);
+            Set<ObjectName> jspMonitorONs =
+                mBeanServer.queryNames(queryJspMonitor, null);
 
             // Special case for the root context
             if (contextName.equals("/")) {
@@ -676,10 +676,11 @@ public class StatusTransformer {
             String onStr = objectName.getDomain() 
                 + ":j2eeType=Servlet,WebModule=" + webModuleName + ",*";
             ObjectName servletObjectName = new ObjectName(onStr);
-            Set set = mBeanServer.queryMBeans(servletObjectName, null);
-            Iterator iterator = set.iterator();
+            Set<ObjectInstance> set =
+                mBeanServer.queryMBeans(servletObjectName, null);
+            Iterator<ObjectInstance> iterator = set.iterator();
             while (iterator.hasNext()) {
-                ObjectInstance oi = (ObjectInstance) iterator.next();
+                ObjectInstance oi = iterator.next();
                 writeWrapper(writer, oi.getObjectName(), mBeanServer, mode);
             }
 
@@ -735,7 +736,7 @@ public class StatusTransformer {
      * Write JSP monitoring information.
      */
     public static void writeJspMonitor(PrintWriter writer,
-                                       Set jspMonitorONs,
+                                       Set<ObjectName> jspMonitorONs,
                                        MBeanServer mBeanServer,
                                        int mode)
             throws Exception {
@@ -743,9 +744,9 @@ public class StatusTransformer {
         int jspCount = 0;
         int jspReloadCount = 0;
 
-        Iterator iter = jspMonitorONs.iterator();
+        Iterator<ObjectName> iter = jspMonitorONs.iterator();
         while (iter.hasNext()) {
-            ObjectName jspMonitorON = (ObjectName) iter.next();
+            ObjectName jspMonitorON = iter.next();
             Object obj = mBeanServer.getAttribute(jspMonitorON, "jspCount");
             jspCount += ((Integer) obj).intValue();
             obj = mBeanServer.getAttribute(jspMonitorON, "jspReloadCount");
@@ -831,7 +832,7 @@ public class StatusTransformer {
 
         char content[] = new char[message.length()];
         message.getChars(0, message.length(), content, 0);
-        StringBuffer result = new StringBuffer(content.length + 50);
+        StringBuilder result = new StringBuilder(content.length + 50);
         for (int i = 0; i < content.length; i++) {
             switch (content[i]) {
             case '<':
