@@ -24,9 +24,8 @@ import java.util.Arrays;
 import java.util.List;
 
 import javax.security.auth.login.LoginContext;
-import javax.security.auth.login.LoginException;
 
-import org.apache.catalina.Realm;
+import org.ietf.jgss.GSSCredential;
 
 
 /**
@@ -34,7 +33,7 @@ import org.apache.catalina.Realm;
  * is available for use by <code>Realm</code> implementations.
  *
  * @author Craig R. McClanahan
- * @version $Id: GenericPrincipal.java 939518 2010-04-30 00:08:58Z kkolinko $
+ * @version $Id: GenericPrincipal.java 1087416 2011-03-31 19:33:04Z markt $
  */
 
 public class GenericPrincipal implements Principal {
@@ -47,13 +46,12 @@ public class GenericPrincipal implements Principal {
      * Construct a new Principal, associated with the specified Realm, for the
      * specified username and password.
      *
-     * @param realm The Realm that owns this Principal
      * @param name The username of the user represented by this Principal
      * @param password Credentials used to authenticate this user
      */
-    public GenericPrincipal(Realm realm, String name, String password) {
+    public GenericPrincipal(String name, String password) {
 
-        this(realm, name, password, null);
+        this(name, password, null);
 
     }
 
@@ -63,14 +61,12 @@ public class GenericPrincipal implements Principal {
      * specified username and password, with the specified role names
      * (as Strings).
      *
-     * @param realm The Realm that owns this principal
      * @param name The username of the user represented by this Principal
      * @param password Credentials used to authenticate this user
      * @param roles List of roles (must be Strings) possessed by this user
      */
-    public GenericPrincipal(Realm realm, String name, String password,
-                            List<String> roles) {
-        this(realm, name, password, roles, null);
+    public GenericPrincipal(String name, String password, List<String> roles) {
+        this(name, password, roles, null);
     }
 
     /**
@@ -78,16 +74,15 @@ public class GenericPrincipal implements Principal {
      * specified username and password, with the specified role names
      * (as Strings).
      *
-     * @param realm The Realm that owns this principal
      * @param name The username of the user represented by this Principal
      * @param password Credentials used to authenticate this user
      * @param roles List of roles (must be Strings) possessed by this user
      * @param userPrincipal - the principal to be returned from the request 
      *        getUserPrincipal call if not null; if null, this will be returned
      */
-    public GenericPrincipal(Realm realm, String name, String password,
-                            List<String> roles, Principal userPrincipal) {
-        this(realm, name, password, roles, userPrincipal, null);
+    public GenericPrincipal(String name, String password, List<String> roles,
+            Principal userPrincipal) {
+        this(name, password, roles, userPrincipal, null);
     }
     
     /**
@@ -95,7 +90,6 @@ public class GenericPrincipal implements Principal {
      * specified username and password, with the specified role names
      * (as Strings).
      *
-     * @param realm The Realm that owns this principal
      * @param name The username of the user represented by this Principal
      * @param password Credentials used to authenticate this user
      * @param roles List of roles (must be Strings) possessed by this user
@@ -104,21 +98,40 @@ public class GenericPrincipal implements Principal {
      * @param loginContext  - If provided, this will be used to log out the user
      *        at the appropriate time
      */
-    public GenericPrincipal(Realm realm, String name, String password,
-                            List<String> roles, Principal userPrincipal,
-                            LoginContext loginContext) {
+    public GenericPrincipal(String name, String password, List<String> roles,
+            Principal userPrincipal, LoginContext loginContext) {
+        this(name, password, roles, userPrincipal, loginContext, null);
+    }
+    
+    /**
+     * Construct a new Principal, associated with the specified Realm, for the
+     * specified username and password, with the specified role names
+     * (as Strings).
+     *
+     * @param name The username of the user represented by this Principal
+     * @param password Credentials used to authenticate this user
+     * @param roles List of roles (must be Strings) possessed by this user
+     * @param userPrincipal - the principal to be returned from the request 
+     *        getUserPrincipal call if not null; if null, this will be returned
+     * @param loginContext  - If provided, this will be used to log out the user
+     *        at the appropriate time
+     * @param gssCredential - If provided, the user&apos;s delegated credentials
+     */
+    public GenericPrincipal(String name, String password, List<String> roles,
+            Principal userPrincipal, LoginContext loginContext,
+            GSSCredential gssCredential) {
         super();
-        this.realm = realm;
         this.name = name;
         this.password = password;
         this.userPrincipal = userPrincipal;
         if (roles != null) {
             this.roles = new String[roles.size()];
-            this.roles = (String[]) roles.toArray(this.roles);
+            this.roles = roles.toArray(this.roles);
             if (this.roles.length > 0)
                 Arrays.sort(this.roles);
         }
         this.loginContext = loginContext;
+        this.gssCredential = gssCredential;
     }
 
 
@@ -130,6 +143,7 @@ public class GenericPrincipal implements Principal {
      */
     protected String name = null;
 
+    @Override
     public String getName() {
         return (this.name);
     }
@@ -146,20 +160,6 @@ public class GenericPrincipal implements Principal {
     }
 
 
-    /**
-     * The Realm with which this Principal is associated.
-     */
-    protected Realm realm = null;
-
-    public Realm getRealm() {
-        return (this.realm);
-    }
-
-    void setRealm( Realm realm ) {
-        this.realm=realm;
-    }
-
-
     /**
      * The set of roles associated with this user.
      */
@@ -190,8 +190,17 @@ public class GenericPrincipal implements Principal {
      */
     protected LoginContext loginContext = null;
 
-    void setLoginContext( LoginContext loginContext ) {
-        this.loginContext=loginContext;
+
+    /**
+     * The user&apos;s delegated credentials.
+     */
+    protected GSSCredential gssCredential = null;
+
+    public GSSCredential getGssCredential() {
+        return this.gssCredential;
+    }
+    protected void setGssCredential(GSSCredential gssCredential) {
+        this.gssCredential = gssCredential;
     }
 
     // --------------------------------------------------------- Public Methods
@@ -217,9 +226,10 @@ public class GenericPrincipal implements Principal {
      * Return a String representation of this object, which exposes only
      * information that should be public.
      */
+    @Override
     public String toString() {
 
-        StringBuffer sb = new StringBuffer("GenericPrincipal[");
+        StringBuilder sb = new StringBuilder("GenericPrincipal[");
         sb.append(this.name);
         sb.append("(");
         for( int i=0;i<roles.length; i++ ) {
@@ -248,4 +258,5 @@ public class GenericPrincipal implements Principal {
     }
 
 
+
 }
