@@ -25,7 +25,7 @@ package org.apache.catalina.security;
  *
  * @author Glenn L. Nielsen
  * @author Jean-Francois Arcand
- * @version $Id: SecurityClassLoad.java 1133586 2011-06-08 22:20:22Z markt $
+ * @version $Id: SecurityClassLoad.java 1174064 2011-09-22 11:23:57Z markt $
  */
 
 public final class SecurityClassLoad {
@@ -87,10 +87,10 @@ public final class SecurityClassLoad {
              "DefaultInstanceManager$3");
         loader.loadClass
             (basePackage +
-             "DefaultInstanceManager$4");
+             "DefaultInstanceManager$AnnotationCacheEntry");
         loader.loadClass
             (basePackage +
-             "DefaultInstanceManager$5");
+             "DefaultInstanceManager$AnnotationCacheEntryType");
         loader.loadClass
             (basePackage +
              "ApplicationHttpRequest$AttributeNamesEnumerator");
@@ -239,6 +239,7 @@ public final class SecurityClassLoad {
     private static final void loadTomcatPackage(ClassLoader loader)
         throws Exception {
         final String basePackage = "org.apache.tomcat.";
+        loader.loadClass(basePackage + "util.buf.HexUtils");
         loader.loadClass(basePackage + "util.buf.StringCache");
         loader.loadClass(basePackage + "util.buf.StringCache$ByteEntry");
         loader.loadClass(basePackage + "util.buf.StringCache$CharEntry");
