@@ -18,8 +18,6 @@
 
 package org.apache.catalina.session;
 
-import java.beans.PropertyChangeEvent;
-import java.beans.PropertyChangeListener;
 import java.io.BufferedInputStream;
 import java.io.BufferedOutputStream;
 import java.io.File;
@@ -34,19 +32,19 @@ import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
 import java.util.ArrayList;
 import java.util.Iterator;
+
 import javax.servlet.ServletContext;
-import org.apache.catalina.Container;
+
 import org.apache.catalina.Context;
-import org.apache.catalina.Globals;
-import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.LifecycleListener;
+import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Loader;
 import org.apache.catalina.Session;
-import org.apache.catalina.util.CustomObjectInputStream;
-import org.apache.catalina.util.LifecycleSupport;
-
 import org.apache.catalina.security.SecurityUtil;
+import org.apache.catalina.util.CustomObjectInputStream;
+import org.apache.juli.logging.Log;
+import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.ExceptionUtils;
 /**
  * Standard implementation of the <b>Manager</b> interface that provides
  * simple session persistence across restarts of this component (such as
@@ -59,33 +57,37 @@ import org.apache.catalina.security.SecurityUtil;
  *
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
- * @version $Id: StandardManager.java 1056768 2011-01-08 19:15:44Z markt $
+ * @version $Id: StandardManager.java 1044944 2010-12-12 22:33:09Z kkolinko $
  */
 
-public class StandardManager
-    extends ManagerBase
-    implements Lifecycle, PropertyChangeListener {
+public class StandardManager extends ManagerBase {
+
+    private final Log log = LogFactory.getLog(StandardManager.class); // must not be static
 
     // ---------------------------------------------------- Security Classes
     private class PrivilegedDoLoad
-        implements PrivilegedExceptionAction {
+        implements PrivilegedExceptionAction<Void> {
 
         PrivilegedDoLoad() {
+            // NOOP
         }
 
-        public Object run() throws Exception{
+        @Override
+        public Void run() throws Exception{
            doLoad();
            return null;
         }
     }
 
     private class PrivilegedDoUnload
-        implements PrivilegedExceptionAction {
+        implements PrivilegedExceptionAction<Void> {
 
         PrivilegedDoUnload() {
+            // NOOP
         }
 
-        public Object run() throws Exception{
+        @Override
+        public Void run() throws Exception{
             doUnload();
             return null;
         }
@@ -102,18 +104,6 @@ public class StandardManager
     protected static final String info = "StandardManager/1.0";
 
 
-    /**
-     * The lifecycle event support for this component.
-     */
-    protected LifecycleSupport lifecycle = new LifecycleSupport(this);
-
-
-    /**
-     * The maximum number of active Sessions allowed, or -1 for no limit.
-     */
-    protected int maxActiveSessions = -1;
-
-
     /**
      * The descriptive name of this Manager implementation (for logging).
      */
@@ -131,58 +121,15 @@ public class StandardManager
     protected String pathname = "SESSIONS.ser";
 
 
-    /**
-     * Has this component been started yet?
-     */
-    protected boolean started = false;
-
-
-    /**
-     * Number of session creations that failed due to maxActiveSessions.
-     */
-    protected int rejectedSessions = 0;
-
-
-    /**
-     * Processing time during session expiration.
-     */
-    protected long processingTime = 0;
-
-
     // ------------------------------------------------------------- Properties
 
 
-    /**
-     * Set the Container with which this Manager has been associated.  If
-     * it is a Context (the usual case), listen for changes to the session
-     * timeout property.
-     *
-     * @param container The associated Container
-     */
-    public void setContainer(Container container) {
-
-        // De-register from the old Container (if any)
-        if ((this.container != null) && (this.container instanceof Context))
-            ((Context) this.container).removePropertyChangeListener(this);
-
-        // Default processing provided by our superclass
-        super.setContainer(container);
-
-        // Register with the new Container (if any)
-        if ((this.container != null) && (this.container instanceof Context)) {
-            setMaxInactiveInterval
-                ( ((Context) this.container).getSessionTimeout()*60 );
-            ((Context) this.container).addPropertyChangeListener(this);
-        }
-
-    }
-
-
     /**
      * Return descriptive information about this Manager implementation and
      * the corresponding version number, in the format
      * <code>&lt;description&gt;/&lt;version&gt;</code>.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -190,51 +137,10 @@ public class StandardManager
     }
 
 
-    /**
-     * Return the maximum number of active Sessions allowed, or -1 for
-     * no limit.
-     */
-    public int getMaxActiveSessions() {
-
-        return (this.maxActiveSessions);
-
-    }
-
-
-    /** Number of session creations that failed due to maxActiveSessions
-     *
-     * @return The count
-     */
-    public int getRejectedSessions() {
-        return rejectedSessions;
-    }
-
-
-    public void setRejectedSessions(int rejectedSessions) {
-        this.rejectedSessions = rejectedSessions;
-    }
-
-
-    /**
-     * Set the maximum number of actives Sessions allowed, or -1 for
-     * no limit.
-     *
-     * @param max The new maximum number of sessions
-     */
-    public void setMaxActiveSessions(int max) {
-
-        int oldMaxActiveSessions = this.maxActiveSessions;
-        this.maxActiveSessions = max;
-        support.firePropertyChange("maxActiveSessions",
-                                   new Integer(oldMaxActiveSessions),
-                                   new Integer(this.maxActiveSessions));
-
-    }
-
-
     /**
      * Return the descriptive short name of this Manager implementation.
      */
+    @Override
     public String getName() {
 
         return (name);
@@ -269,30 +175,6 @@ public class StandardManager
 
     // --------------------------------------------------------- Public Methods
 
-    /**
-     * Construct and return a new session object, based on the default
-     * settings specified by this Manager's properties.  The session
-     * id will be assigned by this method, and available via the getId()
-     * method of the returned session.  If a new session cannot be created
-     * for any reason, return <code>null</code>.
-     *
-     * @exception IllegalStateException if a new session cannot be
-     *  instantiated for any reason
-     */
-    public Session createSession(String sessionId) {
-
-        if ((maxActiveSessions >= 0) &&
-            (sessions.size() >= maxActiveSessions)) {
-            rejectedSessions++;
-            throw new IllegalStateException
-                (sm.getString("standardManager.createSession.ise"));
-        }
-
-        return (super.createSession(sessionId));
-
-    }
-
-
     /**
      * Load any currently active sessions that were previously unloaded
      * to the appropriate persistence mechanism, if any.  If persistence is not
@@ -302,6 +184,7 @@ public class StandardManager
      *  found during the reload
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public void load() throws ClassNotFoundException, IOException {
         if (SecurityUtil.isPackageProtectionEnabled()){
             try{
@@ -346,12 +229,13 @@ public class StandardManager
         if (log.isDebugEnabled())
             log.debug(sm.getString("standardManager.loading", pathname));
         FileInputStream fis = null;
+        BufferedInputStream bis = null;
         ObjectInputStream ois = null;
         Loader loader = null;
         ClassLoader classLoader = null;
         try {
             fis = new FileInputStream(file.getAbsolutePath());
-            BufferedInputStream bis = new BufferedInputStream(fis);
+            bis = new BufferedInputStream(fis);
             if (container != null)
                 loader = container.getLoader();
             if (loader != null)
@@ -371,13 +255,19 @@ public class StandardManager
             return;
         } catch (IOException e) {
             log.error(sm.getString("standardManager.loading.ioe", e), e);
-            if (ois != null) {
+            if (fis != null) {
                 try {
-                    ois.close();
+                    fis.close();
                 } catch (IOException f) {
-                    ;
+                    // Ignore
+                }
+            }
+            if (bis != null) {
+                try {
+                    bis.close();
+                } catch (IOException f) {
+                    // Ignore
                 }
-                ois = null;
             }
             throw e;
         }
@@ -405,37 +295,30 @@ public class StandardManager
                 }
             } catch (ClassNotFoundException e) {
                 log.error(sm.getString("standardManager.loading.cnfe", e), e);
-                if (ois != null) {
-                    try {
-                        ois.close();
-                    } catch (IOException f) {
-                        ;
-                    }
-                    ois = null;
+                try {
+                    ois.close();
+                } catch (IOException f) {
+                    // Ignore
                 }
                 throw e;
             } catch (IOException e) {
                 log.error(sm.getString("standardManager.loading.ioe", e), e);
-                if (ois != null) {
-                    try {
-                        ois.close();
-                    } catch (IOException f) {
-                        ;
-                    }
-                    ois = null;
+                try {
+                    ois.close();
+                } catch (IOException f) {
+                    // Ignore
                 }
                 throw e;
             } finally {
                 // Close the input stream
                 try {
-                    if (ois != null)
-                        ois.close();
+                    ois.close();
                 } catch (IOException f) {
                     // ignored
                 }
 
                 // Delete the persistent storage file
-                if (file != null && file.exists() )
+                if (file.exists() )
                     file.delete();
             }
         }
@@ -452,6 +335,7 @@ public class StandardManager
      *
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public void unload() throws IOException {
         if (SecurityUtil.isPackageProtectionEnabled()){
             try{
@@ -501,41 +385,37 @@ public class StandardManager
             oos = new ObjectOutputStream(new BufferedOutputStream(fos));
         } catch (IOException e) {
             log.error(sm.getString("standardManager.unloading.ioe", e), e);
-            if (oos != null) {
+            if (fos != null) {
                 try {
-                    oos.close();
+                    fos.close();
                 } catch (IOException f) {
-                    ;
+                    // Ignore
                 }
-                oos = null;
             }
             throw e;
         }
 
         // Write the number of active sessions, followed by the details
-        ArrayList list = new ArrayList();
+        ArrayList<StandardSession> list = new ArrayList<StandardSession>();
         synchronized (sessions) {
             if (log.isDebugEnabled())
                 log.debug("Unloading " + sessions.size() + " sessions");
             try {
                 oos.writeObject(new Integer(sessions.size()));
-                Iterator elements = sessions.values().iterator();
+                Iterator<Session> elements = sessions.values().iterator();
                 while (elements.hasNext()) {
                     StandardSession session =
                         (StandardSession) elements.next();
                     list.add(session);
-                    ((StandardSession) session).passivate();
+                    session.passivate();
                     session.writeObjectData(oos);
                 }
             } catch (IOException e) {
                 log.error(sm.getString("standardManager.unloading.ioe", e), e);
-                if (oos != null) {
-                    try {
-                        oos.close();
-                    } catch (IOException f) {
-                        ;
-                    }
-                    oos = null;
+                try {
+                    oos.close();
+                } catch (IOException f) {
+                    // Ignore
                 }
                 throw e;
             }
@@ -544,30 +424,24 @@ public class StandardManager
         // Flush and close the output stream
         try {
             oos.flush();
-            oos.close();
-            oos = null;
-        } catch (IOException e) {
-            if (oos != null) {
-                try {
-                    oos.close();
-                } catch (IOException f) {
-                    ;
-                }
-                oos = null;
+        } finally {
+            try {
+                oos.close();
+            } catch (IOException f) {
+                // Ignore
             }
-            throw e;
         }
 
         // Expire all the sessions we just wrote
         if (log.isDebugEnabled())
             log.debug("Expiring " + list.size() + " persisted sessions");
-        Iterator expires = list.iterator();
+        Iterator<StandardSession> expires = list.iterator();
         while (expires.hasNext()) {
-            StandardSession session = (StandardSession) expires.next();
+            StandardSession session = expires.next();
             try {
                 session.expire(false);
             } catch (Throwable t) {
-                ;
+                ExceptionUtils.handleThrowable(t);
             } finally {
                 session.recycle();
             }
@@ -579,104 +453,50 @@ public class StandardManager
     }
 
 
-    // ------------------------------------------------------ Lifecycle Methods
-
-
     /**
-     * Add a lifecycle event listener to this component.
-     *
-     * @param listener The listener to add
-     */
-    public void addLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.addLifecycleListener(listener);
-
-    }
-
-
-    /**
-     * Get the lifecycle listeners associated with this lifecycle. If this
-     * Lifecycle has no listeners registered, a zero-length array is returned.
-     */
-    public LifecycleListener[] findLifecycleListeners() {
-
-        return lifecycle.findLifecycleListeners();
-
-    }
-
-
-    /**
-     * Remove a lifecycle event listener from this component.
-     *
-     * @param listener The listener to remove
-     */
-    public void removeLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.removeLifecycleListener(listener);
-
-    }
-
-    /**
-     * Prepare for the beginning of active use of the public methods of this
-     * component.  This method should be called after <code>configure()</code>,
-     * and before any of the public methods of the component are utilized.
+     * Start this component and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
      *  that prevents this component from being used
      */
-    public void start() throws LifecycleException {
+    @Override
+    protected synchronized void startInternal() throws LifecycleException {
 
-        if( ! initialized )
-            init();
-
-        // Validate and update our current component state
-        if (started) {
-            return;
-        }
-        lifecycle.fireLifecycleEvent(START_EVENT, null);
-        started = true;
-
-        // Force initialization of the random number generator
-        if (log.isDebugEnabled())
-            log.debug("Force random number initialization starting");
-        String dummy = generateSessionId();
-        if (log.isDebugEnabled())
-            log.debug("Force random number initialization completed");
+        super.startInternal();
 
         // Load unloaded sessions, if any
         try {
             load();
         } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
             log.error(sm.getString("standardManager.managerLoad"), t);
         }
 
+        setState(LifecycleState.STARTING);
     }
 
 
     /**
-     * Gracefully terminate the active use of the public methods of this
-     * component.  This method should be the last one called on a given
-     * instance of this component.
+     * Stop this component and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
-     *  that needs to be reported
+     *  that prevents this component from being used
      */
-    public void stop() throws LifecycleException {
+    @Override
+    protected synchronized void stopInternal() throws LifecycleException {
 
         if (log.isDebugEnabled())
             log.debug("Stopping");
 
-        // Validate and update our current component state
-        if (!started)
-            throw new LifecycleException
-                (sm.getString("standardManager.notStarted"));
-        lifecycle.fireLifecycleEvent(STOP_EVENT, null);
-        started = false;
-
+        setState(LifecycleState.STOPPING);
+        
         // Write out sessions
         try {
             unload();
         } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
             log.error(sm.getString("standardManager.managerUnload"), t);
         }
 
@@ -689,7 +509,7 @@ public class StandardManager
                     session.expire();
                 }
             } catch (Throwable t) {
-                ;
+                ExceptionUtils.handleThrowable(t);
             } finally {
                 // Measure against memory leaking if references to the session
                 // object are kept in a shared field somewhere
@@ -698,40 +518,7 @@ public class StandardManager
         }
 
         // Require a new random number generator if we are restarted
-        this.random = null;
-
-        if( initialized ) {
-            destroy();
-        }
-    }
-
-
-    // ----------------------------------------- PropertyChangeListener Methods
-
-
-    /**
-     * Process property change events from our associated Context.
-     *
-     * @param event The property change event that has occurred
-     */
-    public void propertyChange(PropertyChangeEvent event) {
-
-        // Validate the source of this event
-        if (!(event.getSource() instanceof Context))
-            return;
-        Context context = (Context) event.getSource();
-
-        // Process a relevant property change
-        if (event.getPropertyName().equals("sessionTimeout")) {
-            try {
-                setMaxInactiveInterval
-                    ( ((Integer) event.getNewValue()).intValue()*60 );
-            } catch (NumberFormatException e) {
-                log.error(sm.getString("standardManager.sessionTimeout",
-                                 event.getNewValue().toString()));
-            }
-        }
-
+        super.stopInternal();
     }
 
 
@@ -752,7 +539,7 @@ public class StandardManager
                 ServletContext servletContext =
                     ((Context) container).getServletContext();
                 File tempdir = (File)
-                    servletContext.getAttribute(Globals.WORK_DIR_ATTR);
+                    servletContext.getAttribute(ServletContext.TEMPDIR);
                 if (tempdir != null)
                     file = new File(tempdir, pathname);
             }
