@@ -29,6 +29,7 @@ import java.util.HashMap;
 import javax.el.ELContext;
 import javax.el.ExpressionFactory;
 import javax.el.ValueExpression;
+import javax.servlet.RequestDispatcher;
 import javax.servlet.Servlet;
 import javax.servlet.ServletConfig;
 import javax.servlet.ServletContext;
@@ -47,7 +48,6 @@ import javax.servlet.jsp.el.ExpressionEvaluator;
 import javax.servlet.jsp.el.VariableResolver;
 import javax.servlet.jsp.tagext.BodyContent;
 
-import org.apache.jasper.Constants;
 import org.apache.jasper.compiler.Localizer;
 import org.apache.jasper.el.ELContextImpl;
 import org.apache.jasper.el.ExpressionEvaluatorImpl;
@@ -116,6 +116,7 @@ public class PageContextImpl extends PageContext {
         this.depth = -1;
     }
 
+    @Override
     public void initialize(Servlet servlet, ServletRequest request,
             ServletResponse response, String errorPageURL,
             boolean needsSession, int bufferSize, boolean autoFlush)
@@ -127,8 +128,7 @@ public class PageContextImpl extends PageContext {
 
     private void _initialize(Servlet servlet, ServletRequest request,
             ServletResponse response, String errorPageURL,
-            boolean needsSession, int bufferSize, boolean autoFlush)
-            throws IOException {
+            boolean needsSession, int bufferSize, boolean autoFlush) {
 
         // initialize state
         this.servlet = servlet;
@@ -170,9 +170,11 @@ public class PageContextImpl extends PageContext {
         setAttribute(PAGECONTEXT, this);
         setAttribute(APPLICATION, context);
 
-        isIncluded = request.getAttribute("javax.servlet.include.servlet_path") != null;
+        isIncluded = request.getAttribute(
+                RequestDispatcher.INCLUDE_SERVLET_PATH) != null;
     }
 
+    @Override
     public void release() {
         out = baseOut;
         try {
@@ -209,6 +211,7 @@ public class PageContextImpl extends PageContext {
         }
     }
 
+    @Override
     public Object getAttribute(final String name) {
 
         if (name == null) {
@@ -217,7 +220,9 @@ public class PageContextImpl extends PageContext {
         }
 
         if (SecurityUtil.isPackageProtectionEnabled()) {
-            return AccessController.doPrivileged(new PrivilegedAction() {
+            return AccessController.doPrivileged(
+                    new PrivilegedAction<Object>() {
+                @Override
                 public Object run() {
                     return doGetAttribute(name);
                 }
@@ -232,6 +237,7 @@ public class PageContextImpl extends PageContext {
         return attributes.get(name);
     }
 
+    @Override
     public Object getAttribute(final String name, final int scope) {
 
         if (name == null) {
@@ -240,7 +246,9 @@ public class PageContextImpl extends PageContext {
         }
 
         if (SecurityUtil.isPackageProtectionEnabled()) {
-            return AccessController.doPrivileged(new PrivilegedAction() {
+            return AccessController.doPrivileged(
+                    new PrivilegedAction<Object>() {
+                @Override
                 public Object run() {
                     return doGetAttribute(name, scope);
                 }
@@ -274,6 +282,7 @@ public class PageContextImpl extends PageContext {
         }
     }
 
+    @Override
     public void setAttribute(final String name, final Object attribute) {
 
         if (name == null) {
@@ -282,8 +291,9 @@ public class PageContextImpl extends PageContext {
         }
 
         if (SecurityUtil.isPackageProtectionEnabled()) {
-            AccessController.doPrivileged(new PrivilegedAction() {
-                public Object run() {
+            AccessController.doPrivileged(new PrivilegedAction<Void>() {
+                @Override
+                public Void run() {
                     doSetAttribute(name, attribute);
                     return null;
                 }
@@ -301,6 +311,7 @@ public class PageContextImpl extends PageContext {
         }
     }
 
+    @Override
     public void setAttribute(final String name, final Object o, final int scope) {
 
         if (name == null) {
@@ -309,8 +320,9 @@ public class PageContextImpl extends PageContext {
         }
 
         if (SecurityUtil.isPackageProtectionEnabled()) {
-            AccessController.doPrivileged(new PrivilegedAction() {
-                public Object run() {
+            AccessController.doPrivileged(new PrivilegedAction<Void>() {
+                @Override
+                public Void run() {
                     doSetAttribute(name, o, scope);
                     return null;
                 }
@@ -352,6 +364,7 @@ public class PageContextImpl extends PageContext {
         }
     }
 
+    @Override
     public void removeAttribute(final String name, final int scope) {
 
         if (name == null) {
@@ -359,8 +372,9 @@ public class PageContextImpl extends PageContext {
                     .getMessage("jsp.error.attribute.null_name"));
         }
         if (SecurityUtil.isPackageProtectionEnabled()) {
-            AccessController.doPrivileged(new PrivilegedAction() {
-                public Object run() {
+            AccessController.doPrivileged(new PrivilegedAction<Void>() {
+                @Override
+                public Void run() {
                     doRemoveAttribute(name, scope);
                     return null;
                 }
@@ -397,6 +411,7 @@ public class PageContextImpl extends PageContext {
         }
     }
 
+    @Override
     public int getAttributesScope(final String name) {
 
         if (name == null) {
@@ -405,10 +420,11 @@ public class PageContextImpl extends PageContext {
         }
 
         if (SecurityUtil.isPackageProtectionEnabled()) {
-            return ((Integer) AccessController
-                    .doPrivileged(new PrivilegedAction() {
-                        public Object run() {
-                            return new Integer(doGetAttributeScope(name));
+            return (AccessController
+                    .doPrivileged(new PrivilegedAction<Integer>() {
+                        @Override
+                        public Integer run() {
+                            return Integer.valueOf(doGetAttributeScope(name));
                         }
                     })).intValue();
         } else {
@@ -439,9 +455,12 @@ public class PageContextImpl extends PageContext {
         return 0;
     }
 
+    @Override
     public Object findAttribute(final String name) {
         if (SecurityUtil.isPackageProtectionEnabled()) {
-            return AccessController.doPrivileged(new PrivilegedAction() {
+            return AccessController.doPrivileged(
+                    new PrivilegedAction<Object>() {
+                @Override
                 public Object run() {
                     if (name == null) {
                         throw new NullPointerException(Localizer
@@ -485,11 +504,13 @@ public class PageContextImpl extends PageContext {
         return context.getAttribute(name);
     }
 
+    @Override
     public Enumeration<String> getAttributeNamesInScope(final int scope) {
         if (SecurityUtil.isPackageProtectionEnabled()) {
-            return (Enumeration) AccessController
-                    .doPrivileged(new PrivilegedAction() {
-                        public Object run() {
+            return AccessController.doPrivileged(
+                    new PrivilegedAction<Enumeration<String>>() {
+                        @Override
+                        public Enumeration<String> run() {
                             return doGetAttributeNamesInScope(scope);
                         }
                     });
@@ -498,10 +519,10 @@ public class PageContextImpl extends PageContext {
         }
     }
 
-    private Enumeration doGetAttributeNamesInScope(int scope) {
+    private Enumeration<String> doGetAttributeNamesInScope(int scope) {
         switch (scope) {
         case PAGE_SCOPE:
-            return new Enumerator(attributes.keySet().iterator());
+            return new Enumerator<String>(attributes.keySet().iterator());
 
         case REQUEST_SCOPE:
             return request.getAttributeNames();
@@ -521,6 +542,7 @@ public class PageContextImpl extends PageContext {
         }
     }
 
+    @Override
     public void removeAttribute(final String name) {
 
         if (name == null) {
@@ -529,8 +551,9 @@ public class PageContextImpl extends PageContext {
         }
 
         if (SecurityUtil.isPackageProtectionEnabled()) {
-            AccessController.doPrivileged(new PrivilegedAction() {
-                public Object run() {
+            AccessController.doPrivileged(new PrivilegedAction<Void>() {
+                @Override
+                public Void run() {
                     doRemoveAttribute(name);
                     return null;
                 }
@@ -554,30 +577,32 @@ public class PageContextImpl extends PageContext {
         removeAttribute(name, APPLICATION_SCOPE);
     }
 
+    @Override
     public JspWriter getOut() {
         return out;
     }
 
+    @Override
     public HttpSession getSession() {
         return session;
     }
 
-    public Servlet getServlet() {
-        return servlet;
-    }
-
+    @Override
     public ServletConfig getServletConfig() {
         return config;
     }
 
+    @Override
     public ServletContext getServletContext() {
         return config.getServletContext();
     }
 
+    @Override
     public ServletRequest getRequest() {
         return request;
     }
 
+    @Override
     public ServletResponse getResponse() {
         return response;
     }
@@ -589,6 +614,7 @@ public class PageContextImpl extends PageContext {
      * 
      * @return The Exception associated with this page context, if any.
      */
+    @Override
     public Exception getException() {
         Throwable t = JspRuntimeLibrary.getThrowable(request);
 
@@ -600,6 +626,7 @@ public class PageContextImpl extends PageContext {
         return (Exception) t;
     }
 
+    @Override
     public Object getPage() {
         return servlet;
     }
@@ -608,8 +635,8 @@ public class PageContextImpl extends PageContext {
         String path = relativeUrlPath;
 
         if (!path.startsWith("/")) {
-            String uri = (String) request
-                    .getAttribute("javax.servlet.include.servlet_path");
+            String uri = (String) request.getAttribute(
+                    RequestDispatcher.INCLUDE_SERVLET_PATH);
             if (uri == null)
                 uri = ((HttpServletRequest) request).getServletPath();
             String baseURI = uri.substring(0, uri.lastIndexOf('/'));
@@ -619,18 +646,22 @@ public class PageContextImpl extends PageContext {
         return path;
     }
 
+    @Override
     public void include(String relativeUrlPath) throws ServletException,
             IOException {
         JspRuntimeLibrary
                 .include(request, response, relativeUrlPath, out, true);
     }
 
+    @Override
     public void include(final String relativeUrlPath, final boolean flush)
             throws ServletException, IOException {
         if (SecurityUtil.isPackageProtectionEnabled()) {
             try {
-                AccessController.doPrivileged(new PrivilegedExceptionAction() {
-                    public Object run() throws Exception {
+                AccessController.doPrivileged(
+                        new PrivilegedExceptionAction<Void>() {
+                    @Override
+                    public Void run() throws Exception {
                         doInclude(relativeUrlPath, flush);
                         return null;
                     }
@@ -654,16 +685,21 @@ public class PageContextImpl extends PageContext {
                 flush);
     }
 
+    @Override
+    @Deprecated
     public VariableResolver getVariableResolver() {
         return new VariableResolverImpl(this.getELContext());
     }
 
+    @Override
     public void forward(final String relativeUrlPath) throws ServletException,
             IOException {
         if (SecurityUtil.isPackageProtectionEnabled()) {
             try {
-                AccessController.doPrivileged(new PrivilegedExceptionAction() {
-                    public Object run() throws Exception {
+                AccessController.doPrivileged(
+                        new PrivilegedExceptionAction<Void>() {
+                    @Override
+                    public Void run() throws Exception {
                         doForward(relativeUrlPath);
                         return null;
                     }
@@ -700,23 +736,26 @@ public class PageContextImpl extends PageContext {
         }
 
         final String path = getAbsolutePathRelativeToContext(relativeUrlPath);
-        String includeUri = (String) request
-                .getAttribute(Constants.INC_SERVLET_PATH);
+        String includeUri = (String) request.getAttribute(
+                RequestDispatcher.INCLUDE_SERVLET_PATH);
 
         if (includeUri != null)
-            request.removeAttribute(Constants.INC_SERVLET_PATH);
+            request.removeAttribute(RequestDispatcher.INCLUDE_SERVLET_PATH);
         try {
             context.getRequestDispatcher(path).forward(request, response);
         } finally {
             if (includeUri != null)
-                request.setAttribute(Constants.INC_SERVLET_PATH, includeUri);
+                request.setAttribute(RequestDispatcher.INCLUDE_SERVLET_PATH,
+                        includeUri);
         }
     }
 
+    @Override
     public BodyContent pushBody() {
         return (BodyContent) pushBody(null);
     }
 
+    @Override
     public JspWriter pushBody(Writer writer) {
         depth++;
         if (depth >= outs.length) {
@@ -738,6 +777,7 @@ public class PageContextImpl extends PageContext {
         return outs[depth];
     }
 
+    @Override
     public JspWriter popBody() {
         depth--;
         if (depth >= 0) {
@@ -758,10 +798,13 @@ public class PageContextImpl extends PageContext {
      * Container must return a valid instance of an ExpressionEvaluator that can
      * parse EL expressions.
      */
+    @Override
+    @Deprecated
     public ExpressionEvaluator getExpressionEvaluator() {
         return new ExpressionEvaluatorImpl(this.applicationContext.getExpressionFactory());
     }
 
+    @Override
     public void handlePageException(Exception ex) throws IOException,
             ServletException {
         // Should never be called since handleException() called with a
@@ -769,6 +812,7 @@ public class PageContextImpl extends PageContext {
         handlePageException((Throwable) ex);
     }
 
+    @Override
     public void handlePageException(final Throwable t) throws IOException,
             ServletException {
         if (t == null)
@@ -776,8 +820,10 @@ public class PageContextImpl extends PageContext {
 
         if (SecurityUtil.isPackageProtectionEnabled()) {
             try {
-                AccessController.doPrivileged(new PrivilegedExceptionAction() {
-                    public Object run() throws Exception {
+                AccessController.doPrivileged(
+                        new PrivilegedExceptionAction<Void>() {
+                    @Override
+                    public Void run() throws Exception {
                         doHandlePageException(t);
                         return null;
                     }
@@ -810,13 +856,13 @@ public class PageContextImpl extends PageContext {
              * not been committed (the response will have been committed if the
              * error page is a JSP page).
              */
-            request.setAttribute("javax.servlet.jsp.jspException", t);
-            request.setAttribute("javax.servlet.error.status_code",
+            request.setAttribute(PageContext.EXCEPTION, t);
+            request.setAttribute(RequestDispatcher.ERROR_STATUS_CODE,
                     new Integer(HttpServletResponse.SC_INTERNAL_SERVER_ERROR));
-            request.setAttribute("javax.servlet.error.request_uri",
+            request.setAttribute(RequestDispatcher.ERROR_REQUEST_URI,
                     ((HttpServletRequest) request).getRequestURI());
-            request.setAttribute("javax.servlet.error.servlet_name", config
-                    .getServletName());
+            request.setAttribute(RequestDispatcher.ERROR_SERVLET_NAME,
+                    config.getServletName());
             try {
                 forward(errorPageURL);
             } catch (IllegalStateException ise) {
@@ -825,19 +871,19 @@ public class PageContextImpl extends PageContext {
 
             // The error page could be inside an include.
 
-            Object newException = request
-                    .getAttribute("javax.servlet.error.exception");
+            Object newException =
+                    request.getAttribute(RequestDispatcher.ERROR_EXCEPTION);
 
             // t==null means the attribute was not set.
             if ((newException != null) && (newException == t)) {
-                request.removeAttribute("javax.servlet.error.exception");
+                request.removeAttribute(RequestDispatcher.ERROR_EXCEPTION);
             }
 
             // now clear the error code - to prevent double handling.
-            request.removeAttribute("javax.servlet.error.status_code");
-            request.removeAttribute("javax.servlet.error.request_uri");
-            request.removeAttribute("javax.servlet.error.status_code");
-            request.removeAttribute("javax.servlet.jsp.jspException");
+            request.removeAttribute(RequestDispatcher.ERROR_STATUS_CODE);
+            request.removeAttribute(RequestDispatcher.ERROR_REQUEST_URI);
+            request.removeAttribute(RequestDispatcher.ERROR_SERVLET_NAME);
+            request.removeAttribute(PageContext.EXCEPTION);
 
         } else {
             // Otherwise throw the exception wrapped inside a ServletException.
@@ -852,9 +898,9 @@ public class PageContextImpl extends PageContext {
 
             Throwable rootCause = null;
             if (t instanceof JspException) {
-                rootCause = ((JspException) t).getRootCause();
+                rootCause = ((JspException) t).getCause();
             } else if (t instanceof ELException) {
-                rootCause = ((ELException) t).getRootCause();
+                rootCause = ((ELException) t).getCause();
             }
 
             if (rootCause != null) {
@@ -869,7 +915,7 @@ public class PageContextImpl extends PageContext {
     private static String XmlEscape(String s) {
         if (s == null)
             return null;
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
         for (int i = 0; i < s.length(); i++) {
             char c = s.charAt(i);
             if (c == '<') {
@@ -906,7 +952,7 @@ public class PageContextImpl extends PageContext {
      * @return The result of the evaluation
      */
     public static Object proprietaryEvaluate(final String expression,
-            final Class expectedType, final PageContext pageContext,
+            final Class<?> expectedType, final PageContext pageContext,
             final ProtectedFunctionMapper functionMap, final boolean escape)
             throws ELException {
         Object retValue;
@@ -914,8 +960,9 @@ public class PageContextImpl extends PageContext {
         if (SecurityUtil.isPackageProtectionEnabled()) {
             try {
                 retValue = AccessController
-                        .doPrivileged(new PrivilegedExceptionAction() {
+                        .doPrivileged(new PrivilegedExceptionAction<Object>() {
 
+                            @Override
                             public Object run() throws Exception {
                                 ELContextImpl ctx = (ELContextImpl) pageContext.getELContext();
                                 ctx.setFunctionMapper(new FunctionMapperImpl(functionMap));
@@ -944,6 +991,7 @@ public class PageContextImpl extends PageContext {
         return retValue;
     }
 
+    @Override
     public ELContext getELContext() {
         if (this.elContext == null) {
             this.elContext = this.applicationContext.createELContext(this);
