@@ -18,30 +18,33 @@
 
 package org.apache.naming;
 
+import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.Hashtable;
-import java.util.Enumeration;
-import javax.naming.Context;
-import javax.naming.Name;
-import javax.naming.LinkRef;
+
+import javax.naming.Binding;
 import javax.naming.CompositeName;
+import javax.naming.Context;
+import javax.naming.InitialContext;
+import javax.naming.LinkRef;
+import javax.naming.Name;
+import javax.naming.NameAlreadyBoundException;
+import javax.naming.NameClassPair;
+import javax.naming.NameNotFoundException;
 import javax.naming.NameParser;
-import javax.naming.Referenceable;
-import javax.naming.Reference;
 import javax.naming.NamingEnumeration;
 import javax.naming.NamingException;
-import javax.naming.NameAlreadyBoundException;
-import javax.naming.NameNotFoundException;
 import javax.naming.NotContextException;
-import javax.naming.InitialContext;
 import javax.naming.OperationNotSupportedException;
+import javax.naming.Reference;
+import javax.naming.Referenceable;
 import javax.naming.spi.NamingManager;
 
 /**
  * Catalina JNDI Context implementation.
  *
  * @author Remy Maucherat
- * @version $Revision: 891201 $ $Date: 2009-12-16 12:37:40 +0100 (Mi, 16. Dez 2009) $
+ * @version $Id: NamingContext.java 1163633 2011-08-31 14:20:17Z markt $
  */
 public class NamingContext implements Context {
 
@@ -55,7 +58,7 @@ public class NamingContext implements Context {
     protected static final NameParser nameParser = new NameParserImpl();
 
 
-    private static org.apache.juli.logging.Log log =
+    private static final org.apache.juli.logging.Log log =
         org.apache.juli.logging.LogFactory.getLog(NamingContext.class);
 
 
@@ -65,17 +68,17 @@ public class NamingContext implements Context {
     /**
      * Builds a naming context using the given environment.
      */
-    public NamingContext(Hashtable env, String name) 
+    public NamingContext(Hashtable<String,Object> env, String name) 
         throws NamingException {
-        this.bindings = new HashMap();
-        this.env = new Hashtable();
+        this.bindings = new HashMap<String,NamingEntry>();
+        this.env = new Hashtable<String,Object>();
         // FIXME ? Could be put in the environment ?
         this.name = name;
         // Populating the environment hashtable
         if (env != null ) {
-            Enumeration envEntries = env.keys();
+            Enumeration<String> envEntries = env.keys();
             while (envEntries.hasMoreElements()) {
-                String entryName = (String) envEntries.nextElement();
+                String entryName = envEntries.nextElement();
                 addToEnvironment(entryName, env.get(entryName));
             }
         }
@@ -85,7 +88,8 @@ public class NamingContext implements Context {
     /**
      * Builds a naming context using the given environment.
      */
-    public NamingContext(Hashtable env, String name, HashMap bindings) 
+    public NamingContext(Hashtable<String,Object> env, String name,
+            HashMap<String,NamingEntry> bindings) 
         throws NamingException {
         this(env, name);
         this.bindings = bindings;
@@ -98,19 +102,19 @@ public class NamingContext implements Context {
     /**
      * Environment.
      */
-    protected Hashtable env;
+    protected Hashtable<String,Object> env;
 
 
     /**
      * The string manager for this package.
      */
-    protected StringManager sm = StringManager.getManager(Constants.Package);
+    protected static final StringManager sm = StringManager.getManager(Constants.Package);
 
 
     /**
      * Bindings in this Context.
      */
-    protected HashMap bindings;
+    protected HashMap<String,NamingEntry> bindings;
 
 
     /**
@@ -135,6 +139,7 @@ public class NamingContext implements Context {
      * @return the object bound to name
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Object lookup(Name name)
         throws NamingException {
         return lookup(name, true);
@@ -148,6 +153,7 @@ public class NamingContext implements Context {
      * @return the object bound to name
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Object lookup(String name)
         throws NamingException {
         return lookup(new CompositeName(name), true);
@@ -162,10 +168,11 @@ public class NamingContext implements Context {
      * @param name the name to bind; may not be empty
      * @param obj the object to bind; possibly null
      * @exception NameAlreadyBoundException if name is already bound
-     * @exception InvalidAttributesException if object did not supply all 
-     * mandatory attributes
+     * @exception javax.naming.directory.InvalidAttributesException if object
+     * did not supply all mandatory attributes
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void bind(Name name, Object obj)
         throws NamingException {
         bind(name, obj, false);
@@ -178,10 +185,11 @@ public class NamingContext implements Context {
      * @param name the name to bind; may not be empty
      * @param obj the object to bind; possibly null
      * @exception NameAlreadyBoundException if name is already bound
-     * @exception InvalidAttributesException if object did not supply all 
-     * mandatory attributes
+     * @exception javax.naming.directory.InvalidAttributesException if object
+     * did not supply all mandatory attributes
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void bind(String name, Object obj)
         throws NamingException {
         bind(new CompositeName(name), obj);
@@ -199,10 +207,11 @@ public class NamingContext implements Context {
      * 
      * @param name the name to bind; may not be empty
      * @param obj the object to bind; possibly null
-     * @exception InvalidAttributesException if object did not supply all 
-     * mandatory attributes
+     * @exception javax.naming.directory.InvalidAttributesException if object
+     * did not supply all mandatory attributes
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void rebind(Name name, Object obj)
         throws NamingException {
         bind(name, obj, true);
@@ -214,10 +223,11 @@ public class NamingContext implements Context {
      * 
      * @param name the name to bind; may not be empty
      * @param obj the object to bind; possibly null
-     * @exception InvalidAttributesException if object did not supply all 
-     * mandatory attributes
+     * @exception javax.naming.directory.InvalidAttributesException if object
+     * did not supply all mandatory attributes
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void rebind(String name, Object obj)
         throws NamingException {
         rebind(new CompositeName(name), obj);
@@ -238,6 +248,7 @@ public class NamingContext implements Context {
      * exist
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void unbind(Name name)
         throws NamingException {
         checkWritable();
@@ -248,7 +259,7 @@ public class NamingContext implements Context {
             throw new NamingException
                 (sm.getString("namingContext.invalidName"));
         
-        NamingEntry entry = (NamingEntry) bindings.get(name.get(0));
+        NamingEntry entry = bindings.get(name.get(0));
         
         if (entry == null) {
             throw new NameNotFoundException
@@ -277,6 +288,7 @@ public class NamingContext implements Context {
      * exist
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void unbind(String name)
         throws NamingException {
         unbind(new CompositeName(name));
@@ -294,6 +306,7 @@ public class NamingContext implements Context {
      * @exception NameAlreadyBoundException if newName is already bound
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void rename(Name oldName, Name newName)
         throws NamingException {
         Object value = lookup(oldName);
@@ -311,6 +324,7 @@ public class NamingContext implements Context {
      * @exception NameAlreadyBoundException if newName is already bound
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void rename(String oldName, String newName)
         throws NamingException {
         rename(new CompositeName(oldName), new CompositeName(newName));
@@ -330,7 +344,8 @@ public class NamingContext implements Context {
      * this context. Each element of the enumeration is of type NameClassPair.
      * @exception NamingException if a naming exception is encountered
      */
-    public NamingEnumeration list(Name name)
+    @Override
+    public NamingEnumeration<NameClassPair> list(Name name)
         throws NamingException {
         // Removing empty parts
         while ((!name.isEmpty()) && (name.get(0).length() == 0))
@@ -339,7 +354,7 @@ public class NamingContext implements Context {
             return new NamingContextEnumeration(bindings.values().iterator());
         }
         
-        NamingEntry entry = (NamingEntry) bindings.get(name.get(0));
+        NamingEntry entry = bindings.get(name.get(0));
         
         if (entry == null) {
             throw new NameNotFoundException
@@ -363,7 +378,8 @@ public class NamingContext implements Context {
      * this context. Each element of the enumeration is of type NameClassPair.
      * @exception NamingException if a naming exception is encountered
      */
-    public NamingEnumeration list(String name)
+    @Override
+    public NamingEnumeration<NameClassPair> list(String name)
         throws NamingException {
         return list(new CompositeName(name));
     }
@@ -382,7 +398,8 @@ public class NamingContext implements Context {
      * Each element of the enumeration is of type Binding.
      * @exception NamingException if a naming exception is encountered
      */
-    public NamingEnumeration listBindings(Name name)
+    @Override
+    public NamingEnumeration<Binding> listBindings(Name name)
         throws NamingException {
         // Removing empty parts
         while ((!name.isEmpty()) && (name.get(0).length() == 0))
@@ -391,7 +408,7 @@ public class NamingContext implements Context {
             return new NamingContextBindingsEnumeration(bindings.values().iterator(), this);
         }
         
-        NamingEntry entry = (NamingEntry) bindings.get(name.get(0));
+        NamingEntry entry = bindings.get(name.get(0));
         
         if (entry == null) {
             throw new NameNotFoundException
@@ -415,7 +432,8 @@ public class NamingContext implements Context {
      * Each element of the enumeration is of type Binding.
      * @exception NamingException if a naming exception is encountered
      */
-    public NamingEnumeration listBindings(String name)
+    @Override
+    public NamingEnumeration<Binding> listBindings(String name)
         throws NamingException {
         return listBindings(new CompositeName(name));
     }
@@ -446,6 +464,7 @@ public class NamingContext implements Context {
      * @exception NotContextException if the name is bound but does not name 
      * a context, or does not name a context of the appropriate type
      */
+    @Override
     public void destroySubcontext(Name name)
         throws NamingException {
         
@@ -457,7 +476,7 @@ public class NamingContext implements Context {
             throw new NamingException
                 (sm.getString("namingContext.invalidName"));
         
-        NamingEntry entry = (NamingEntry) bindings.get(name.get(0));
+        NamingEntry entry = bindings.get(name.get(0));
         
         if (entry == null) {
             throw new NameNotFoundException
@@ -493,6 +512,7 @@ public class NamingContext implements Context {
      * @exception NotContextException if the name is bound but does not name 
      * a context, or does not name a context of the appropriate type
      */
+    @Override
     public void destroySubcontext(String name)
         throws NamingException {
         destroySubcontext(new CompositeName(name));
@@ -508,10 +528,11 @@ public class NamingContext implements Context {
      * @param name the name of the context to create; may not be empty
      * @return the newly created context
      * @exception NameAlreadyBoundException if name is already bound
-     * @exception InvalidAttributesException if creation of the subcontext 
-     * requires specification of mandatory attributes
+     * @exception javax.naming.directory.InvalidAttributesException if creation
+     * of the sub-context requires specification of mandatory attributes
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Context createSubcontext(Name name)
         throws NamingException {
         checkWritable();
@@ -529,10 +550,11 @@ public class NamingContext implements Context {
      * @param name the name of the context to create; may not be empty
      * @return the newly created context
      * @exception NameAlreadyBoundException if name is already bound
-     * @exception InvalidAttributesException if creation of the subcontext 
-     * requires specification of mandatory attributes
+     * @exception javax.naming.directory.InvalidAttributesException if creation 
+     * of the sub-context requires specification of mandatory attributes
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Context createSubcontext(String name)
         throws NamingException {
         return createSubcontext(new CompositeName(name));
@@ -549,6 +571,7 @@ public class NamingContext implements Context {
      * (if any).
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Object lookupLink(Name name)
         throws NamingException {
         return lookup(name, false);
@@ -564,6 +587,7 @@ public class NamingContext implements Context {
      * (if any).
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Object lookupLink(String name)
         throws NamingException {
         return lookup(new CompositeName(name), false);
@@ -584,6 +608,7 @@ public class NamingContext implements Context {
      * components
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public NameParser getNameParser(Name name)
         throws NamingException {
 
@@ -615,6 +640,7 @@ public class NamingContext implements Context {
      * components
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public NameParser getNameParser(String name)
         throws NamingException {
         return getNameParser(new CompositeName(name));
@@ -636,6 +662,7 @@ public class NamingContext implements Context {
      * @return the composition of prefix and name
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Name composeName(Name name, Name prefix)
         throws NamingException {
         prefix = (Name) prefix.clone();
@@ -651,6 +678,7 @@ public class NamingContext implements Context {
      * @return the composition of prefix and name
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public String composeName(String name, String prefix)
         throws NamingException {
         return prefix + "/" + name;
@@ -666,6 +694,7 @@ public class NamingContext implements Context {
      * @param propVal the value of the property to add; may not be null
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Object addToEnvironment(String propName, Object propVal)
         throws NamingException {
         return env.put(propName, propVal);
@@ -679,6 +708,7 @@ public class NamingContext implements Context {
      * may not be null
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Object removeFromEnvironment(String propName)
         throws NamingException {
         return env.remove(propName);
@@ -695,7 +725,8 @@ public class NamingContext implements Context {
      * @return the environment of this context; never null
      * @exception NamingException if a naming exception is encountered
      */
-    public Hashtable getEnvironment()
+    @Override
+    public Hashtable<?,?> getEnvironment()
         throws NamingException {
         return env;
     }
@@ -711,8 +742,9 @@ public class NamingContext implements Context {
      * 
      * @exception NamingException if a naming exception is encountered
      */
-    public void close()
-        throws NamingException {
+    @Override
+    public void close() throws NamingException {
+        checkWritable();
         env.clear();
     }
 
@@ -734,6 +766,7 @@ public class NamingContext implements Context {
      * not have the notion of a full name
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public String getNameInNamespace()
         throws NamingException {
         throw  new OperationNotSupportedException
@@ -764,7 +797,7 @@ public class NamingContext implements Context {
             return new NamingContext(env, this.name, bindings);
         }
         
-        NamingEntry entry = (NamingEntry) bindings.get(name.get(0));
+        NamingEntry entry = bindings.get(name.get(0));
         
         if (entry == null) {
             throw new NameNotFoundException
@@ -792,11 +825,16 @@ public class NamingContext implements Context {
                 try {
                     Object obj = NamingManager.getObjectInstance
                         (entry.value, name, this, env);
-                    if (obj != null) {
-                        entry.value = obj;
-                        entry.type = NamingEntry.ENTRY;
+                    if(entry.value instanceof ResourceRef) {
+                        boolean singleton = Boolean.parseBoolean(
+                                    (String) ((ResourceRef) entry.value).get(
+                                        "singleton").getContent());
+                        if (singleton) {
+                            entry.type = NamingEntry.ENTRY;
+                            entry.value = obj;
+                        }
                     }
-                    return obj;
+                    return obj; 
                 } catch (NamingException e) {
                     throw e;
                 } catch (Exception e) {
@@ -821,8 +859,8 @@ public class NamingContext implements Context {
      * @param obj the object to bind; possibly null
      * @param rebind if true, then perform a rebind (ie, overwrite)
      * @exception NameAlreadyBoundException if name is already bound
-     * @exception InvalidAttributesException if object did not supply all 
-     * mandatory attributes
+     * @exception javax.naming.directory.InvalidAttributesException if object
+     * did not supply all mandatory attributes
      * @exception NamingException if a naming exception is encountered
      */
     protected void bind(Name name, Object obj, boolean rebind)
@@ -836,7 +874,7 @@ public class NamingContext implements Context {
             throw new NamingException
                 (sm.getString("namingContext.invalidName"));
         
-        NamingEntry entry = (NamingEntry) bindings.get(name.get(0));
+        NamingEntry entry = bindings.get(name.get(0));
         
         if (name.size() > 1) {
             if (entry == null) {
