@@ -19,8 +19,11 @@
 package org.apache.naming;
 
 import java.util.Hashtable;
+
+import javax.naming.Binding;
 import javax.naming.Context;
 import javax.naming.Name;
+import javax.naming.NameClassPair;
 import javax.naming.NameParser;
 import javax.naming.NamingEnumeration;
 import javax.naming.NamingException;
@@ -29,7 +32,7 @@ import javax.naming.NamingException;
  * Catalina JNDI Context implementation.
  *
  * @author Remy Maucherat
- * @version $Revision: 795749 $ $Date: 2009-07-20 12:40:29 +0200 (Mo, 20. Jul 2009) $
+ * @version $Id: SelectorContext.java 1026057 2010-10-21 16:26:50Z markt $
  */
 
 public class SelectorContext implements Context {
@@ -56,7 +59,7 @@ public class SelectorContext implements Context {
     public static final String IC_PREFIX = "IC_";
 
 
-    private static org.apache.juli.logging.Log log =
+    private static final org.apache.juli.logging.Log log =
         org.apache.juli.logging.LogFactory.getLog(SelectorContext.class);
 
     // ----------------------------------------------------------- Constructors
@@ -65,7 +68,7 @@ public class SelectorContext implements Context {
     /**
      * Builds a Catalina selector context using the given environment.
      */
-    public SelectorContext(Hashtable env) {
+    public SelectorContext(Hashtable<String,Object> env) {
         this.env = env;
     }
 
@@ -73,7 +76,8 @@ public class SelectorContext implements Context {
     /**
      * Builds a Catalina selector context using the given environment.
      */
-    public SelectorContext(Hashtable env, boolean initialContext) {
+    public SelectorContext(Hashtable<String,Object> env,
+            boolean initialContext) {
         this(env);
         this.initialContext = initialContext;
     }
@@ -85,13 +89,13 @@ public class SelectorContext implements Context {
     /**
      * Environment.
      */
-    protected Hashtable env;
+    protected Hashtable<String,Object> env;
 
 
     /**
      * The string manager for this package.
      */
-    protected StringManager sm = StringManager.getManager(Constants.Package);
+    protected static final StringManager sm = StringManager.getManager(Constants.Package);
 
 
     /**
@@ -116,6 +120,7 @@ public class SelectorContext implements Context {
      * @return the object bound to name
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Object lookup(Name name)
         throws NamingException {
         
@@ -138,6 +143,7 @@ public class SelectorContext implements Context {
      * @return the object bound to name
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Object lookup(String name)
         throws NamingException {
 
@@ -160,11 +166,13 @@ public class SelectorContext implements Context {
      * 
      * @param name the name to bind; may not be empty
      * @param obj the object to bind; possibly null
-     * @exception NameAlreadyBoundException if name is already bound
-     * @exception InvalidAttributesException if object did not supply all 
-     * mandatory attributes
+     * @exception javax.naming.NameAlreadyBoundException if name is already
+     * bound
+     * @exception javax.naming.InvalidAttributesException if object did not
+     * supply all mandatory attributes
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void bind(Name name, Object obj)
         throws NamingException {
         getBoundContext().bind(parseName(name), obj);
@@ -176,11 +184,13 @@ public class SelectorContext implements Context {
      * 
      * @param name the name to bind; may not be empty
      * @param obj the object to bind; possibly null
-     * @exception NameAlreadyBoundException if name is already bound
-     * @exception InvalidAttributesException if object did not supply all 
-     * mandatory attributes
+     * @exception javax.naming.NameAlreadyBoundException if name is already
+     * bound
+     * @exception javax.naming.InvalidAttributesException if object did not
+     * supply all mandatory attributes
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void bind(String name, Object obj)
         throws NamingException {
         getBoundContext().bind(parseName(name), obj);
@@ -198,10 +208,11 @@ public class SelectorContext implements Context {
      * 
      * @param name the name to bind; may not be empty
      * @param obj the object to bind; possibly null
-     * @exception InvalidAttributesException if object did not supply all 
-     * mandatory attributes
+     * @exception javax.naming.InvalidAttributesException if object did not
+     * supply all mandatory attributes
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void rebind(Name name, Object obj)
         throws NamingException {
         getBoundContext().rebind(parseName(name), obj);
@@ -213,10 +224,11 @@ public class SelectorContext implements Context {
      * 
      * @param name the name to bind; may not be empty
      * @param obj the object to bind; possibly null
-     * @exception InvalidAttributesException if object did not supply all 
-     * mandatory attributes
+     * @exception javax.naming.InvalidAttributesException if object did not
+     * supply all mandatory attributes
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void rebind(String name, Object obj)
         throws NamingException {
         getBoundContext().rebind(parseName(name), obj);
@@ -233,10 +245,11 @@ public class SelectorContext implements Context {
      * NameNotFoundException if any of the intermediate contexts do not exist. 
      * 
      * @param name the name to bind; may not be empty
-     * @exception NameNotFoundException if an intermediate context does not 
-     * exist
+     * @exception javax.naming NameNotFoundException if an intermediate context
+     * does not exist
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void unbind(Name name)
         throws NamingException {
         getBoundContext().unbind(parseName(name));
@@ -247,10 +260,11 @@ public class SelectorContext implements Context {
      * Unbinds the named object.
      * 
      * @param name the name to bind; may not be empty
-     * @exception NameNotFoundException if an intermediate context does not 
-     * exist
+     * @exception javax.naming NameNotFoundException if an intermediate context
+     * does not exist
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void unbind(String name)
         throws NamingException {
         getBoundContext().unbind(parseName(name));
@@ -265,9 +279,11 @@ public class SelectorContext implements Context {
      * 
      * @param oldName the name of the existing binding; may not be empty
      * @param newName the name of the new binding; may not be empty
-     * @exception NameAlreadyBoundException if newName is already bound
+     * @exception javax.naming.NameAlreadyBoundException if name is already
+     * bound
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void rename(Name oldName, Name newName)
         throws NamingException {
         getBoundContext().rename(parseName(oldName), parseName(newName));
@@ -280,9 +296,11 @@ public class SelectorContext implements Context {
      * 
      * @param oldName the name of the existing binding; may not be empty
      * @param newName the name of the new binding; may not be empty
-     * @exception NameAlreadyBoundException if newName is already bound
+     * @exception javax.naming.NameAlreadyBoundException if name is already
+     * bound
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void rename(String oldName, String newName)
         throws NamingException {
         getBoundContext().rename(parseName(oldName), parseName(newName));
@@ -302,7 +320,8 @@ public class SelectorContext implements Context {
      * this context. Each element of the enumeration is of type NameClassPair.
      * @exception NamingException if a naming exception is encountered
      */
-    public NamingEnumeration list(Name name)
+    @Override
+    public NamingEnumeration<NameClassPair> list(Name name)
         throws NamingException {
         
         if (log.isDebugEnabled()) {
@@ -323,7 +342,8 @@ public class SelectorContext implements Context {
      * this context. Each element of the enumeration is of type NameClassPair.
      * @exception NamingException if a naming exception is encountered
      */
-    public NamingEnumeration list(String name)
+    @Override
+    public NamingEnumeration<NameClassPair> list(String name)
         throws NamingException {
         
         if (log.isDebugEnabled()) {
@@ -348,7 +368,8 @@ public class SelectorContext implements Context {
      * Each element of the enumeration is of type Binding.
      * @exception NamingException if a naming exception is encountered
      */
-    public NamingEnumeration listBindings(Name name)
+    @Override
+    public NamingEnumeration<Binding> listBindings(Name name)
         throws NamingException {
 
         if (log.isDebugEnabled()) {
@@ -369,7 +390,8 @@ public class SelectorContext implements Context {
      * Each element of the enumeration is of type Binding.
      * @exception NamingException if a naming exception is encountered
      */
-    public NamingEnumeration listBindings(String name)
+    @Override
+    public NamingEnumeration<Binding> listBindings(String name)
         throws NamingException {
 
         if (log.isDebugEnabled()) {
@@ -401,11 +423,12 @@ public class SelectorContext implements Context {
      * the foreign context's "native" naming system.
      * 
      * @param name the name of the context to be destroyed; may not be empty
-     * @exception NameNotFoundException if an intermediate context does not 
-     * exist
-     * @exception NotContextException if the name is bound but does not name 
-     * a context, or does not name a context of the appropriate type
+     * @exception javax.naming NameNotFoundException if an intermediate context
+     * does not exist
+     * @exception javax.naming.NotContextException if the name is bound but does
+     * not name a context, or does not name a context of the appropriate type
      */
+    @Override
     public void destroySubcontext(Name name)
         throws NamingException {
         getBoundContext().destroySubcontext(parseName(name));
@@ -416,11 +439,12 @@ public class SelectorContext implements Context {
      * Destroys the named context and removes it from the namespace.
      * 
      * @param name the name of the context to be destroyed; may not be empty
-     * @exception NameNotFoundException if an intermediate context does not 
-     * exist
-     * @exception NotContextException if the name is bound but does not name 
-     * a context, or does not name a context of the appropriate type
+     * @exception javax.naming NameNotFoundException if an intermediate context
+     * does not exist
+     * @exception javax.naming.NotContextException if the name is bound but does
+     * not name a context, or does not name a context of the appropriate type
      */
+    @Override
     public void destroySubcontext(String name)
         throws NamingException {
         getBoundContext().destroySubcontext(parseName(name));
@@ -435,11 +459,13 @@ public class SelectorContext implements Context {
      * 
      * @param name the name of the context to create; may not be empty
      * @return the newly created context
-     * @exception NameAlreadyBoundException if name is already bound
-     * @exception InvalidAttributesException if creation of the subcontext 
-     * requires specification of mandatory attributes
+     * @exception javax.naming.NameAlreadyBoundException if name is already
+     * bound
+     * @exception javax.naming.InvalidAttributesException if creation of the
+     * sub-context requires specification of mandatory attributes
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Context createSubcontext(Name name)
         throws NamingException {
         return getBoundContext().createSubcontext(parseName(name));
@@ -451,11 +477,13 @@ public class SelectorContext implements Context {
      * 
      * @param name the name of the context to create; may not be empty
      * @return the newly created context
-     * @exception NameAlreadyBoundException if name is already bound
-     * @exception InvalidAttributesException if creation of the subcontext 
-     * requires specification of mandatory attributes
+     * @exception javax.naming.NameAlreadyBoundException if name is already
+     * bound
+     * @exception javax.naming.InvalidAttributesException if creation of the
+     * sub-context requires specification of mandatory attributes
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Context createSubcontext(String name)
         throws NamingException {
         return getBoundContext().createSubcontext(parseName(name));
@@ -472,6 +500,7 @@ public class SelectorContext implements Context {
      * (if any).
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Object lookupLink(Name name)
         throws NamingException {
 
@@ -493,6 +522,7 @@ public class SelectorContext implements Context {
      * (if any).
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Object lookupLink(String name)
         throws NamingException {
 
@@ -519,6 +549,7 @@ public class SelectorContext implements Context {
      * components
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public NameParser getNameParser(Name name)
         throws NamingException {
         return getBoundContext().getNameParser(parseName(name));
@@ -533,6 +564,7 @@ public class SelectorContext implements Context {
      * components
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public NameParser getNameParser(String name)
         throws NamingException {
         return getBoundContext().getNameParser(parseName(name));
@@ -554,10 +586,11 @@ public class SelectorContext implements Context {
      * @return the composition of prefix and name
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Name composeName(Name name, Name prefix)
         throws NamingException {
-        prefix = (Name) prefix.clone();
-        return prefix.addAll(name);
+        Name prefixClone = (Name) prefix.clone();
+        return prefixClone.addAll(name);
     }
 
 
@@ -569,6 +602,7 @@ public class SelectorContext implements Context {
      * @return the composition of prefix and name
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public String composeName(String name, String prefix)
         throws NamingException {
         return prefix + "/" + name;
@@ -584,6 +618,7 @@ public class SelectorContext implements Context {
      * @param propVal the value of the property to add; may not be null
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Object addToEnvironment(String propName, Object propVal)
         throws NamingException {
         return getBoundContext().addToEnvironment(propName, propVal);
@@ -597,6 +632,7 @@ public class SelectorContext implements Context {
      * may not be null
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public Object removeFromEnvironment(String propName)
         throws NamingException {
         return getBoundContext().removeFromEnvironment(propName);
@@ -613,7 +649,8 @@ public class SelectorContext implements Context {
      * @return the environment of this context; never null
      * @exception NamingException if a naming exception is encountered
      */
-    public Hashtable getEnvironment()
+    @Override
+    public Hashtable<?,?> getEnvironment()
         throws NamingException {
         return getBoundContext().getEnvironment();
     }
@@ -629,6 +666,7 @@ public class SelectorContext implements Context {
      * 
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public void close()
         throws NamingException {
         getBoundContext().close();
@@ -648,10 +686,11 @@ public class SelectorContext implements Context {
      * OperationNotSupportedException is thrown.
      * 
      * @return this context's name in its own namespace; never null
-     * @exception OperationNotSupportedException if the naming system does 
-     * not have the notion of a full name
+     * @exception javax.naming.OperationNotSupportedException if the naming
+     * system does not have the notion of a full name
      * @exception NamingException if a naming exception is encountered
      */
+    @Override
     public String getNameInNamespace()
         throws NamingException {
         return prefix;
