@@ -116,9 +116,9 @@ public class TestRegistration extends TomcatBaseTest {
 
         final Tomcat tomcat = getTomcatInstance();
         final File contextDir = new File(getTemporaryDirectory(), "webappFoo");
-        if (!contextDir.exists()) {
-            if (!contextDir.mkdir())
-                fail("Failed to create: [" + contextDir.toString() + "]");
+        addDeleteOnTearDown(contextDir);
+        if (!contextDir.mkdirs() && !contextDir.isDirectory()) {
+            fail("Failed to create: [" + contextDir.toString() + "]");
         }
         tomcat.addContext(contextName, contextDir.getAbsolutePath());
         tomcat.start();
@@ -172,9 +172,9 @@ public class TestRegistration extends TomcatBaseTest {
         tomcat.getEngine().addChild(host);
 
         final File contextDir2 = new File(getTemporaryDirectory(), "webappFoo2");
-        if (!contextDir2.exists()) {
-            if (!contextDir2.mkdir())
-                fail("Failed to create: [" + contextDir2.toString() + "]");
+        addDeleteOnTearDown(contextDir2);
+        if (!contextDir2.mkdirs() && !contextDir2.isDirectory()) {
+            fail("Failed to create: [" + contextDir2.toString() + "]");
         }
         tomcat.addContext(host, contextName + "2", contextDir2.getAbsolutePath());
         
